/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      app.c
*
* \brief     uC/OS-MPU QUICKSTART APPLICATON
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/Examples/SP_QuickStart/app.c#1 $
*
* \details  This file contains the main function, the application task functions and the
*           exception/fault callback functions.
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \mainpage  Quickstart Demo Application for uC/OS-MPU
*
* \details   This quickstart application includes multiple modules. Each module will focus a
*            single safety topic.
*
* \details   ## Application Modules
*
* \details   The application is divided in different modules which are described in the following
*            chapters.
*
* \details   ### Startup Module
*            The startup module shows the typical startup as described in the "Safety Manual"
*            and is divided into
*             - the main entry function (RTOS is not running) and
*             - the safe application "Start Task" (RTOS is running).
*
* \details   ### Task Demo Module
*            The task demo module illustrates the use of tasks, assigned to memory spaces
*            (called 'process'). A task is an endless loop, which is scheduled by the RTOS.
*
* \details   All tasks within a process are executed with equal access permissions. Most
*            interesting is the same layout for memory write permissions.
*            When setting up two processes with different writable areas, the freedom of
*            interference by overwriting memory is guaranteed.
*
* \details   The demo task module assigns two tasks to a process, called 'Safe Process' and
*            two task to a process, called 'Unsafe Process':
*             - Safe Process: Application "Start Task" and "Control Task"
*             - Unsafe Process: Application "LED Task" and "UART Task"
*
* \note      The names 'Safe Process' and 'Unsafe Process' are just names of the process.
*            The RTOS will not change the safety capability of a task by itself.
*
* \details   ### RTOS Monitoring Module
*            The RTOS monitoring module describes the usage of the included RTOS monitoring
*            features. The integration of these RTOS monitoring features is mandatory to ensure
*            the correct operation of the task scheduler and the guaranteed freedom of
*            interference between multiple processes:
*             - monitoring task stacks
*             - monitoring memory protection settings.
*
* \details   ### Software Watchdog Module
*            The software watchdog module describes the usage of the included RTOS software timer
*            for ensuring the execution of a function within a specified time interval.
*            For this demo, the background monitoring function is checked.
*
* \details   ## Quickstart Application Overview
*            The following figure shows an overview of the application components of the
*            quickstart application.
*
* @startuml
*            package "Safe Process" {
*              component [<&loop-circular> Start Task\n(Error Task)] as App_StartTask #Coral
*              component [<&loop-circular> Control Task] as App_CtrlTask #LightSteelBlue
*              interface "<&flag> Error\nMailbox" as App_ErrMBox #Coral
*              interface "<&flag> Control\nSemaphore" as App_CtrlSem #LightSteelBlue
*              App_ErrMBox -- [App_StartTask]
*              App_CtrlSem -- [App_CtrlTask]
*
*              package "Application Callback Functions" {
*                interface "WD Timer\nCallback" as App_SwWdTmrCallback
*                interface "Idle\nHook" as App_TaskIdleHook
*                interface "Task Switch\nHook" as App_TaskSwHook
*                interface "TimeTick\nHook" as App_TimeTickHook
*              }
*
*              interface "Stack Pattern\nCheck" as App_MonStkPattern
*              interface "Stack Pointer\nCheck" as App_MonTaskSw
*              interface "MPU Config\nCheck" as SPMonMpuCfgChk
*              interface "Trigger Watchdog" as App_SwWdTmrTrigger
*              App_MonStkPattern -- [Monitoring]
*              App_MonTaskSw -- [Monitoring]
*              SPMonMpuCfgChk -- [Monitoring]
*            }
*
*            package "Unsafe Process" {
*              component [<&loop-circular> LED Task] as App_LedTask #LightSteelBlue
*              component [<&loop-circular> UART Task] as App_UartTask #LightSteelBlue
*              interface "<&flag> Command\nQueue" as App_CmdQ #LightSteelBlue
*              App_CmdQ - [App_LedTask]
*            }
*
*            package "RTOS Process" {
*              component [<&loop-circular> Timer Task] as OSTmr_Task
*              component [<&loop-circular> Idle Task] as OS_TaskIdle
*              component [Scheduler]
*              component [Time Tick] as OSTimeTick
*            }
*
*            [OSTmr_Task] --> [App_SwWdTmrCallback] : call
*            [OS_TaskIdle] --> [App_TaskIdleHook] : call
*            [Scheduler] --> [App_TaskSwHook] : call
*            [OSTimeTick] --> [App_TimeTickHook] : call
*
*            [App_TaskIdleHook] --> [App_MonStkPattern] : call
*            [App_TaskIdleHook] --> [SPMonMpuCfgChk] : call
*            [App_TaskSwHook] --> [App_MonTaskSw] : call
*
*            [Monitoring] --> [App_ErrMBox] : Post Error
*            [Monitoring] --> [App_SwWdTmrTrigger]
*            [App_SwWdTmrCallback] --> [App_ErrMBox] : Post Error
*            [App_CtrlTask] --> [App_ErrMBox] : Post Error
*
*            [App_TimeTickHook] --> [App_CtrlSem] : Post
*            [App_CtrlTask] --> [App_CmdQ] : Post Command
*
*            legend
*              Legend:
*              <&loop-circular> : OS Task
*              <&flag> : OS Event (e.g. semaphore, mailbox, queue)
*            end legend
* @enduml
*/
/*------------------------------------------------------------------------------------------------*/

/*
****************************************************************************************************
*                                             INCLUDES
****************************************************************************************************
*/

#include "app.h"

/*
****************************************************************************************************
*                                              GLOBALS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \var       App_StartStk
* \ingroup   APP_STARTUP
* \brief     Stack memory for Start Task
* \details   Because the Start Task will be assigned to the "Safe Process", the memory is
*            located to the safe process memory area using the macro SAFE_MEM_SEC.
*
* \var       App_ErrMBox
* \ingroup   APP_STARTUP
* \details   Error Mailbox handle for Start Task
*/
/*------------------------------------------------------------------------------------------------*/
SAFE_MEM_SEC OS_STK    App_StartStk[APP_START_STK_SZ];
SAFE_MEM_SEC OS_EVENT *App_ErrMBox;


SAFE_MEM_DECLARE;                                     /* Get section memory labels from linker    */
UNSAFE_MEM_DECLARE;                                   /* Get section memory labels from linker    */

/*
****************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
****************************************************************************************************
*/

static void App_StartTask   (void *p_arg);


/*
****************************************************************************************************
*                                        STARTUP FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  APP_STARTUP    Startup Module
*
* \details   The startup modules implements the typical startup as described in the "Safety Manual"
*            and is divided into the Main Entry Function (RTOS is not running) and the safe
*            application "Start Task" (RTOS is running).
*/
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Main Entry Function
* \ingroup   APP_STARTUP
*
* \details   The entry function is responsible for the uC/OS-MPU system startup:
*             1. Perform low level and CPU initialization.
*             2. Initialize uC/OS-MPU system by calling OSInit().
*             3. Create process for application "Start Task".
*             4. Create the application "Start Task" with highest priority.
*             5. Start uC/OS-MPU system by calling OSStart().
*
* \note      This implementation covers Safety Manual requirement SR_0032.
*/
/*------------------------------------------------------------------------------------------------*/
void main (void)
{
    SP_PROCESS_T *phdl;                               /* Local: process handle                    */
                                                      /*------------------------------------------*/
    BSPLowInit();                                     /* Initialize Hardware Essential Parts      */
    OSInit();                                         /* initialize uC/OS-MPU realtime kernel     */
                                                      /*------------------------------------------*/
    phdl = SPProcessCreate(SAFE_MEM_START,            /* Create process space for safe memory     */
                           SAFE_MEM_END);

    App_TaskCreate(App_StartTask,                     /* Create Application Start Task            */
                   APP_START_PRIO,
                   App_StartStk,
                   APP_START_STK_SZ,
                   (void *)phdl,
                   (INT8U *)"Start Task");

    OSStart();                                        /* Start uC/OS-MPU realtime kernel          */
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     High Priority Application Start Task
* \ingroup   APP_STARTUP
*
* \details   This task is the first task (with the highest task priority) running after OSStart().
*            This task
*             1. Do the high level BSP initialization.
*             2. Change to non-privileged (user) mode.
*             3. Create RTOS resources used in this application.
*             4. Call OSSafetyCriticalStart() to indicate that all initialization has been
*                completed and that kernel objects are no longer allowed to be created.
*             5. Install and start a hardware timer for the OS System Time Tick.
*                Note, to prevent the usage of a RTOS event handle in App_TimeTickHook() before
*                it is created, the OS System Time Tick timer is started only after completed
*                initialization.
*
* \details   In this application the Start Task is re-used as Error Handler Task. For this
*            purpose a mailbox (Error Mailbox) is used on that this task is pending. Because this
*            task has the highest priority, the task is scheduled/executed immediately after an
*            error message is posted to the Error Mailbox from another task.
*
* \details   Task endless loop:
*               - Wait for error message in the Error Mailbox
*               - Print error message to serial interface
*               - Stop CPU and enter safe state by calling OS_SAFETY_CRITICAL_EXCEPTION()
*
* \note      This task function is executed in privileged mode until SP_SET_USR_MODE() is called.
*
* \note      This implementation covers Safety Manual requirements SR_0033, SR_0034 and SR_0035.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_StartTask (void *p_arg)
{
    INT8U  err = OS_ERR_NONE;
    void  *msg;
                                                      /*------------------------------------------*/
    UNUSED(p_arg);
                                                      /*------------------------------------------*/
                                                      /* TASK IS RUNNING IN PRIVILEGED MODE       */
                                                      /*------------------------------------------*/
    SP_SET_USR_MODE();                                /* Switch to non-privileged (user) mode     */
                                                      /* TASK IS RUNNING IN NON-PRIVILEGED MODE   */
                                                      /*------------------------------------------*/
    App_Init();                                       /* Create application tasks and events      */
    App_SwWdTmrInit();                                /* Initialize software watchdog             */

    App_ErrMBox = OSMboxCreate((void*)0);             /* Create Error Mailbox                     */
    if (App_ErrMBox == (OS_EVENT *)0) {
        OS_SAFETY_CRITICAL_EXCEPTION();               /* Stop in case of an error                 */
    }
                                                      /*------------------------------------------*/
    OSSafetyCriticalStart();                          /* Indicate completed initialization        */

    SP_DO_PRIVIL(
         RTOS_Tick_Init();                            /* Config and start OS System Time Tick     */
    )
                                                      /*------------------------------------------*/
                                                      /* Use Start Task as Error Handler Task     */
                                                      /* with highest priority                    */
    OSTaskNameSet(OS_PRIO_SELF,
                  (INT8U *)"Error Handler Task",
                  &err);
    for (;;) {                                        /* Endless Task Loop                        */
        msg = OSMboxPend(App_ErrMBox, 0u, &err);
        if (err == OS_ERR_NONE) {
            App_PrintString("\n\n  !!! ERROR OCCURRED !!!!\n  Error ");
            App_PrintUInt(APP_ERR_UNPACK_ERR(msg));
            App_PrintString(" reported in task with priority ");
            App_PrintUInt(APP_ERR_UNPACK_PRIO(msg));
            App_PrintString(".\n  Application stopped!\n\n");
        }
        OS_SAFETY_CRITICAL_EXCEPTION();               /* Stop CPU in safety state                 */
    }
}

/*
****************************************************************************************************
*                                      APPLICATION FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Helper function to create a RTOS task
* \ingroup   APP_DEMO
*
* \details   This function is used to simplify the creation of an application task. For this
*            demo application a few task parameter are not used or can be set to fixed values:
*             - p_arg : not used, set to 0
*             - ptos, pbos : calculated depending on the stack growth, the given stack array and
*               the given stack size
*             - id : not used, set to the priority value
*             - opt : set to OS_TASK_OPT_NONE
*
* \param     task      Pointer to the task's code
* \param     prio      Task's priority.
* \param     stk       Task's stack memory array
* \param     stkSize   Size of the stack in number of OS_STK elements
* \param     phdl      Process handle the task will be assigned to.
* \param     pname     ASCII string that contains the name of the task
*/
/*------------------------------------------------------------------------------------------------*/
void App_TaskCreate (void (*task)(void *p_arg), INT8U prio,
                     OS_STK stk[], INT32U stkSize, void *phdl,
                     INT8U *pname)
{
    INT8U   err;
    OS_STK *ptos;
    OS_STK *pbos;
                                                      /*------------------------------------------*/
#if OS_STK_GROWTH == 1                                /* Calculate the stack boundaries depending */
    ptos = &stk[stkSize - 1];                         /* on the stack growth and the stack size   */
    pbos = &stk[0];
#else
    ptos = &stk[0];
    pbos = &stk[stkSize - 1];
#endif
                                                      /*------------------------------------------*/
    err = OSTaskCreateExt(task,
                          (void *)0,
                          ptos,
                          prio,
                          prio,
                          pbos,
                          stkSize,
                          phdl,
                          OS_TASK_OPT_NONE );

    if (err != OS_ERR_NONE) {                         /* see, if task can't be created            */
        OS_SAFETY_CRITICAL_EXCEPTION();               /* Fatal Error: stop CPU in safety state    */
    }

#if (OS_TASK_NAME_EN > 0u)
    OSTaskNameSet(prio, pname, &err);                 /* Set the task's name                      */
#endif
}
