/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      app_mon.c
*
* \brief     uC/OS-MPU QUICKSTART APPLICATON MONITORING
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/Examples/SP_QuickStart/app_mon.c#1 $
*
* \details  This file contains the monitoring functions for the quickstart application.
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  APP_MON     RTOS Monitoring Module
*
* \details   The application implements examples for the following monitors:
*             - Stack pointer check of the current task during the task switch
*             - Stack pattern check for the RTOS tasks during IDLE task is running
*             - Check of the memory protection settings during IDLE task is running
*
* \details   After the task stacks and the memory protection settings are checked,
*            the Software Watchdog timer is triggered.
*
* \note      The diagnostic timing and the fault reaction is application specific.
*            For this demo, the diagnostic time is 0.5s (see #APP_WDT_TIMEOUT) and the
*            fault reaction is activating an error task.
*
* @startuml
*            package "Safe Process" {
*              component [<&loop-circular> Error Task] as App_StartTask #Coral
*              interface "<&flag> Error Mailbox" as App_ErrMBox #Coral
*              App_ErrMBox -- [App_StartTask]
*
*              package "Application Callback Functions" {
*                interface "Idle Hook" as App_TaskIdleHook
*                interface "Task Switch Hook" as App_TaskSwHook
*              }
*
*              interface "Stack Pattern\nCheck" as App_MonStkPattern
*              interface "Stack Pointer\nCheck" as App_MonTaskSw
*              interface "MPU Config\nCheck" as SPMonMpuCfgChk
*              App_MonStkPattern -- [Monitoring]
*              App_MonTaskSw -- [Monitoring]
*              SPMonMpuCfgChk -- [Monitoring]
*            }
*
*            package "RTOS Process" {
*              component [<&loop-circular> Idle Task] as OS_TaskIdle
*              component [Scheduler]
*            }
*
*            [OS_TaskIdle] --> [App_TaskIdleHook] : call
*            [Scheduler] --> [App_TaskSwHook] : call
*
*            [App_TaskIdleHook] --> [App_MonStkPattern] : call
*            [App_TaskIdleHook] --> [SPMonMpuCfgChk] : call
*            [App_TaskSwHook] --> [App_MonTaskSw] : call
*
*            [Monitoring] --> [App_ErrMBox] : Post Error
* @enduml
*/
/*------------------------------------------------------------------------------------------------*/

/*
****************************************************************************************************
*                                             INCLUDES
****************************************************************************************************
*/

#include "app.h"


/*
****************************************************************************************************
*                                        MONITORING FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Task Switch Monitor
* \ingroup   APP_MON
*
* \details   This function is called during a task switch from the function App_TaskSwHook() to
*            check if the stack pointer of the high ready task is within the the valid range of the
*            stack memory.
*
* \details   If the stack pointer of the high ready task is out of the range defined in the stack
*            monitor structure, the error APP_ERR_STK_PTR is posted into the Error Mailbox.
*            With this post, the error handler task is readied by the scheduler.
*
* \note      To keep this demo clear, each failure is posted to the Error Mailbox what results
*            in an immediate switch to the Application Start (Error) Task. If the failure is
*            detected in the stack of Application Start Task, this could be a problem. In this
*            case, the CPU is stopped immediately by calling OS_SAFETY_CRITICAL_EXCEPTION()
*
* \note      The high ready task is the highest priority task that is ready to run. The task
*            switch starts/resumes this task.
*/
/*------------------------------------------------------------------------------------------------*/
void App_MonTaskSw (void)
{
    SP_MON_ERR_T  err;                                /* Local: error code                        */
    SP_MON_STK_T *stkMon;                             /* Local: stack monitoring data handle      */
    OS_STK       *stkPtr;
                                                      /*------------------------------------------*/
    stkMon = SPThreadStkMonGet(OSPrioHighRdy);
    stkPtr = OSTCBHighRdy->OSTCBStkPtr;
                                                      /*------------------------------------------*/
#if FIT_STK_PTR_EN > 0                                /* Set FIT_STK_PTR_EN to 1 to inject a      */
    if (OSTime > FIT_DELAY) {                         /* stack pointer fault after the defined    */
        stkPtr = (OS_STK *)0;                         /* delay                                    */
    }
#endif
                                                      /*------------------------------------------*/
    err = SPMonStkPtrChk(stkMon, stkPtr);
    if (err != SP_MON_ERR_NONE ) {
        if (OSPrioHighRdy == APP_START_PRIO) {        /* Check if failure in stack of Start Task  */
            OS_SAFETY_CRITICAL_EXCEPTION();           /* Yes, stop CPU in safety state            */
        }
        (void)OSMboxPostOpt(App_ErrMBox,
                            APP_ERR_PACK_HIGHRDY(APP_ERR_STK_PTR),
                            OS_POST_OPT_NONE);
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Idle Monitor
* \ingroup   APP_MON
*
* \details   This function is called from the IDLE task within the function App_TaskIdleHook().
*            The function is used to do background monitoring of the task stacks and the memory
*            protection settings.
*
* \details   The function checks the patterns in the stacks of the tasks, listed in the
*            application task priority table App_TaskPrioTbl[]. In case of an error, the error
*            APP_ERR_STK_PATTERN is posted into the Error Mailbox.
*
* \details   If an error in the memory protection settings is detected, the error APP_ERR_MPU_CFG
*            is posted into the Error Mailbox.
*
* \details   After the monitoring is finished, the SW Watchdog Timer is triggered by calling
*            App_SwWdTmrTrigger().
*
* \note      To keep this demo clear, each failure is posted to the Error Mailbox what results
*            in an immediate switch to the Application Start (Error) Task. If the failure is
*            detected in the stack of Application Start Task, this could be a problem. In this
*            case, the CPU is stopped immediately by calling OS_SAFETY_CRITICAL_EXCEPTION()
*/
/*------------------------------------------------------------------------------------------------*/
void App_MonIdle (void)
{
    SP_MON_STK_T *mondata;
    SP_MON_ERR_T  err;
    INT8U         prio;
    INT32U        i;
                                                      /*------------------------------------------*/
#if FIT_STK_PATTERN_EN > 0                            /* Set FIT_STK_PATTERN_EN to 1 to inject a  */
        if (OSTime > FIT_DELAY) {                     /* stack pattern fault after the defined    */
            App_CtrlStk[0] = 0u;                      /* delay                                    */
            App_CtrlStk[APP_CTRL_STK_SZ-1] = 0u;
        }
#endif
                                                      /*------------------------------------------*/
    for (i = 0u; i < App_TaskPrioTblSize; i++) {      /* Check task stack patterns                */
        prio    = App_TaskPrioTbl[i];
        mondata = SPThreadStkMonGet(prio);
        err     = SPMonStkPatternChk(mondata);
        if (err != SP_MON_ERR_NONE) {
            if (prio == APP_START_PRIO) {             /* Check if failure in stack of Start Task  */
                OS_SAFETY_CRITICAL_EXCEPTION();       /* Yes, stop CPU in safety state            */
            }
            (void)OSMboxPostOpt(App_ErrMBox,
                                APP_ERR_PACK(APP_ERR_STK_PATTERN, prio),
                                OS_POST_OPT_NONE);
        }
    }
                                                      /*------------------------------------------*/
#if MON_MPU_CFG_CHK_EN > 0
    err = SPMonMpuCfgChk();                           /* Check MPU configuration                  */
    if (err != SP_MON_ERR_NONE) {
        (void)OSMboxPostOpt(App_ErrMBox,
                            APP_ERR_PACK_CUR(APP_ERR_MPU_CFG),
                            OS_POST_OPT_NONE);
    }
#endif
                                                      /*------------------------------------------*/
    App_SwWdTmrTrigger();                             /* Trigger the SW Watchdog Timer            */
}
