/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      app_swwd.c
*
* \brief     uC/OS-MPU QUICKSTART APPLICATON SW WATCHDOG
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/Examples/SP_QuickStart/app_swwd.c#1 $
*
* \details  This file contains the application software watchog implementation.
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  APP_SWWD    Software Watchdog Module
*
* \details   In this application the included RTOS software timer module is used to realize a
*            simple software watchdog that ensures the execution of a function within a specified
*            time interval.
*
* \details   With the RTOS timer module it is possible to execute callback functions periodically
*            or once after a specified time. To realize the software watchdog a one shot timer
*            is used. This timer is started
*             - once after the initialization and
*             - every time the monitoring functions in the idle task hook function are finished
*
* \details   Restarting the timer in the monitored function before the timer delay is elapsed
*            prevents the execution of the timer callback function.
*
* @startuml
*            package "Safe Process" {
*              component [<&loop-circular> Error Task] as App_StartTask #Coral
*              component [Monitoring]
*              interface "<&flag> Error Mailbox" as App_ErrMBox #Coral
*              App_ErrMBox -- [App_StartTask]
*
*              package "Application Callback Functions" {
*                interface "WD Timer Callback" as App_SwWdTmrCallback
*              }
*
*              interface "Trigger Watchdog" as App_SwWdTmrTrigger
*              note bottom of App_SwWdTmrTrigger : Restarts the WD Timer
*            }
*
*            package "RTOS Process" {
*              component [<&loop-circular> Timer Task] as OSTmr_Task
*            }
*
*            [OSTmr_Task] --> [App_SwWdTmrCallback] : call
*
*            [Monitoring] --> [App_SwWdTmrTrigger]
*            [App_SwWdTmrCallback] --> [App_ErrMBox] : Post Error
* @enduml
*
* \note      To guarantee that the timer callback is executed immediately after the watchdog
*            timeout, the priority of the RTOS timer task must be higher than the priorities of
*            the monitored tasks.
*/
/*------------------------------------------------------------------------------------------------*/

/*
****************************************************************************************************
*                                             INCLUDES
****************************************************************************************************
*/

#include "app.h"

/*
****************************************************************************************************
*                                             DEFINES
****************************************************************************************************
*/

#define SAFE_MEM_SEC        SP_MEM(10)
#define SAFE_MEM_START      SP_MEM_START(10)
#define SAFE_MEM_END        SP_MEM_END(10)
#define SAFE_MEM_DECLARE    SP_MEM_DECLARE(10)

#define UNSAFE_MEM_SEC      SP_MEM(11)
#define UNSAFE_MEM_START    SP_MEM_START(11)
#define UNSAFE_MEM_END      SP_MEM_END(11)
#define UNSAFE_MEM_DECLARE  SP_MEM_DECLARE(11)

/*------------------------------------------------------------------------------------------------*/
/*!
* \def       APP_WDT_TIMEOUT
* \ingroup   APP_SWWD
* \brief     Software watchdog timeout value.
* \details   The software watchdog timeout value has a resolution of 1/OS_TMR_CFG_TICKS_PER_SEC
*            seconds. In this application the timeout is set to 0.5 seconds.
*/
/*------------------------------------------------------------------------------------------------*/
#define APP_WDT_TIMEOUT     (OS_TMR_CFG_TICKS_PER_SEC / 2u)

/*
****************************************************************************************************
*                                              GLOBALS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \var       App_SwWdTmr
* \ingroup   APP_SWWD
* \details   Timer handle used for Software Watchdog
*/
/*------------------------------------------------------------------------------------------------*/
SAFE_MEM_SEC   OS_TMR   *App_SwWdTmr;


/*
****************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
****************************************************************************************************
*/

static void App_SwWdTmrCallback (void *ptmr, void *parg);


/*
****************************************************************************************************
*                                        SOFTWARE WATCHDOG
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Software Watchdog Initialization
* \ingroup   APP_SWWD
*
* \details   This function creates the one shot timer "SW Watchdog Tmr" that is used for the
*            software watchdog. After successful creation of the timer, the timer is started
*            the first time by calling App_SwWdTmrTrigger().
*/
/*------------------------------------------------------------------------------------------------*/
void App_SwWdTmrInit (void)
{
    INT8U err = OS_ERR_NONE;
                                                      /*------------------------------------------*/
    App_SwWdTmr = OSTmrCreate(APP_WDT_TIMEOUT,
                              0u,
                              OS_TMR_OPT_ONE_SHOT,
                              App_SwWdTmrCallback,
                              (void *)0,
                              (INT8U *)"SW Watchdog Tmr",
                              &err);

    if (App_SwWdTmr == (OS_TMR *)0) {
        OS_SAFETY_CRITICAL_EXCEPTION();
    } else {
        App_SwWdTmrTrigger();                         /* Start software watchdog timer            */
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Software Watchdog Trigger
* \ingroup   APP_SWWD
*
* \details   This function is used to start resp. restart the software watchdog timer.
*/
/*------------------------------------------------------------------------------------------------*/
void App_SwWdTmrTrigger (void)
{
    BOOLEAN started;
    INT8U   err = OS_ERR_NONE;
                                                      /*------------------------------------------*/
    started = OSTmrStart(App_SwWdTmr,
                         &err);

    if (started != OS_TRUE) {
        (void)OSMboxPostOpt(App_ErrMBox,
                            APP_ERR_PACK_CUR(APP_ERR_WDT),
                            OS_POST_OPT_NONE);
    }
                                                      /*------------------------------------------*/
#if (FIT_SWWD_EN > 0)                                 /* Set FIT_SWWD_EN to 1 to inject a         */
    while (OSTime > FIT_DELAY) {                      /* software watchdog fault after the        */
    }                                                 /* defined delay                            */
#endif
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Software Watchdog Callback Function
* \ingroup   APP_SWWD
*
* \details   This is the callback function of the one shot software watchdog timer. The callback
*            function is called from the RTOS timer module if the timer delay is elapsed.
*            Because the software watchdog timer is restarted periodically, this callback
*            function is only executed in case of an error (missing/delayed call of
*            App_SwWdTmrTrigger()).
*
* \details   The function posts the error APP_ERR_WDT to the Error Mailbox.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_SwWdTmrCallback (void *ptmr, void *parg)
{
    (void)ptmr;
    (void)parg;
                                                      /*------------------------------------------*/
    (void)OSMboxPostOpt(App_ErrMBox,
                        APP_ERR_PACK_CUR(APP_ERR_WDT),
                        OS_POST_OPT_NONE);
}
