/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      app_task.c
*
* \brief     uC/OS-MPU QUICKSTART APPLICATON TASKS
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/Examples/SP_QuickStart/app_task.c#1 $
*
* \details  This file contains the main function, the application task functions and the
*           exception/fault callback functions.
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  APP_DEMO    Task Demo Module
*
* \details   The task demo module uses two tasks (Control and LED Task) to show a simple queue
*            communication between two tasks assigned to two different processes.
*
* \details   A third task "UART Task" is used to print diagnostic data to the serial interface.
*
* @startuml
*            package "Safe Process" {
*              component [<&loop-circular> Error Task] as App_StartTask #Coral
*              component [<&loop-circular> Control Task] as App_CtrlTask #LightSteelBlue
*              interface "<&flag> Error Mailbox" as App_ErrMBox #Coral
*              interface "<&flag> Control\nSemaphore" as App_CtrlSem #LightSteelBlue
*              App_ErrMBox -- [App_StartTask]
*              App_CtrlSem - [App_CtrlTask]
*
*              package "Application Callback Functions" {
*                interface "TimeTick Hook" as App_TimeTickHook
*              }
*            }
*
*            package "Unsafe Process" {
*              component [<&loop-circular> LED Task] as App_LedTask #LightSteelBlue
*              component [<&loop-circular> UART Task] as App_UartTask #LightSteelBlue
*              interface "<&flag> Command\nQueue" as App_CmdQ #LightSteelBlue
*              App_CmdQ - [App_LedTask]
*            }
*
*            [App_TimeTickHook] --> [App_CtrlSem] : Post
*            [App_CtrlTask] --> [App_ErrMBox] : Post Error
*            [App_CtrlTask] --> [App_CmdQ] : Post Command
* @enduml
*/
/*------------------------------------------------------------------------------------------------*/

/*
****************************************************************************************************
*                                             INCLUDES
****************************************************************************************************
*/

#include "app.h"

/*
****************************************************************************************************
*                                             DEFINES
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \def       APP_CMD_Q_SZ
* \ingroup   APP_DEMO
* \brief     Command Queue message buffer size.
* \details   Length of the Command Queue.
*/
/*------------------------------------------------------------------------------------------------*/
#define APP_CMD_Q_SZ        8u


/*
****************************************************************************************************
*                                              GLOBALS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \var       App_CtrlStk
* \ingroup   APP_DEMO
* \brief     Stack memory for Control Task
* \details   Because the Control Task will be assigned to the "Safe Process", the memory is
*            located to the safe process memory area using the macro SAFE_MEM_SEC.
*
* \var       App_LedStk
* \ingroup   APP_DEMO
* \brief     Stack memory for LED Task
* \details   Because the LED Task will be assigned to the "Unsafe Process", the memory is
*            located to the unsafe process memory area using the macro UNSAFE_MEM_SEC.
*
* \var       App_UartStk
* \ingroup   APP_DEMO
* \brief     Stack memory for UART Task
* \details   Because the UART Task will be assigned to the "Unsafe Process", the memory is
*            located to the unsafe process memory area using the macro UNSAFE_MEM_SEC.
*
* \var       App_CtrlSem
* \ingroup   APP_DEMO
* \brief     Semaphore handle for Control Task
* \details   Because the semaphore will be created from the Start Task, the event is
*            located to the safe process memory area using the macro SAFE_MEM_SEC. For the further
*            usage of the semaphore handle only read access is needed. This is permitted for all
*            tasks.
*
* \var       App_CmdQ
* \ingroup   APP_DEMO
* \brief     Command Queue handle for LED Task
* \details   Because the queue will be created from the Start Task, the event is
*            located to the safe process memory area using the macro SAFE_MEM_SEC. For the further
*            usage of the semaphore handle only read access is needed. This is permitted for all
*            tasks.
*
* \var       App_CmdQMsg
* \ingroup   APP_DEMO
* \brief     Command Queue message buffer
* \details   The queue message buffer is filled from the RTOS by calling the queue post function.
*            Because the RTOS have write access to the overall memory, the queue message buffer
*            is put into the protected memory area (no linker macro used) to prevent unintended
*            writes to the buffer.
*/
/*------------------------------------------------------------------------------------------------*/
SAFE_MEM_SEC   OS_STK App_CtrlStk[APP_CTRL_STK_SZ];
UNSAFE_MEM_SEC OS_STK App_LedStk[APP_LED_STK_SZ];
UNSAFE_MEM_SEC OS_STK App_UartStk[APP_UART_STK_SZ];

SAFE_MEM_SEC   OS_EVENT *App_CtrlSem;
SAFE_MEM_SEC   OS_EVENT *App_CmdQ;

void *App_CmdQMsg[APP_CMD_Q_SZ];

/*------------------------------------------------------------------------------------------------*/
/*!
* \var       App_TaskPrioTbl
* \ingroup   APP_MON
* \details   Application task priority table
*
* \var       App_TaskPrioTblSize
* \ingroup   APP_MON
* \details   Size (number of entries) of application task priority table.
*/
/*------------------------------------------------------------------------------------------------*/
const INT8U App_TaskPrioTbl[] = {
    OS_TASK_IDLE_PRIO,
    OS_TASK_TMR_PRIO,
    APP_START_PRIO,
    APP_CTRL_PRIO,
    APP_LED_PRIO,
    APP_UART_PRIO
};

const INT32U App_TaskPrioTblSize = sizeof(App_TaskPrioTbl) / sizeof(INT8U);

/*------------------------------------------------------------------------------------------------*/

UNSAFE_MEM_DECLARE;                                   /* Get section memory labels from linker    */

/*
****************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
****************************************************************************************************
*/

static void App_CtrlTask (void *p_arg);
static void App_LedTask  (void *p_arg);
static void App_UartTask (void *p_arg);


/*
****************************************************************************************************
*                                      APPLICATION FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Creation of RTOS resources
* \ingroup   APP_DEMO
*
* \details   This function is called from the "Start Task" to create the RTOS resources used
*            by the task demo:
*             - Control Task (note that the process handle used during creation of this task
*               is 0 and therefore the task is assigned to the process of the current task
*               (App_StartTask()).
*             - UART Task
*             - LED Task
*             - Semaphore for triggering the Control Task
*             - Command Queue to send commands to the LED Task
*/
/*------------------------------------------------------------------------------------------------*/
void App_Init (void)
{
    SP_PROCESS_T *phdl;                               /* Local: process handle                    */
                                                      /*------------------------------------------*/
    App_TaskCreate(App_CtrlTask,                      /* Create Control Task                      */
                   APP_CTRL_PRIO,
                   App_CtrlStk,
                   APP_CTRL_STK_SZ,
                   (void *)0,                         /* assign to process of current task        */
                   (INT8U *)"Control Task");
                                                      /*------------------------------------------*/
    phdl = SPProcessCreate(UNSAFE_MEM_START,          /* Create process space for unsafe memory   */
                           UNSAFE_MEM_END);

    App_TaskCreate(App_LedTask,                       /* Create LED Task                          */
                   APP_LED_PRIO,
                   App_LedStk,
                   APP_LED_STK_SZ,
                   (void *)phdl,                      /* assign to unsafe process                 */
                   (INT8U *)"LED Task");

    App_TaskCreate(App_UartTask,                      /* Create Serial Communication Task         */
                   APP_UART_PRIO,
                   App_UartStk,
                   APP_UART_STK_SZ,
                   (void *)phdl,                      /* assign to unsafe process                 */
                   (INT8U *)"UART Task");
                                                      /*------------------------------------------*/
    App_CmdQ    = OSQCreate(&App_CmdQMsg[0], APP_CMD_Q_SZ);
    App_CtrlSem = OSSemCreate(0u);

    if ((App_CmdQ    == (OS_EVENT *)0) ||             /* Check successful creation of events      */
        (App_CtrlSem == (OS_EVENT *)0)) {
        OS_SAFETY_CRITICAL_EXCEPTION();               /* Stop in case of an error                 */
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Control Task
* \ingroup   APP_DEMO
*
* \details   This task waits for the control semaphore that is triggered cyclic in the
*            App_TimeTickHook() function. Every time the semaphore is received successful, a
*            command is posted to the Command Queue.
*
* \details   This will be repeated in an endless loop.
*
* \details   In case of an error, the error code APP_ERR_CTRL_TASK is posted to the Error Mailbox.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_CtrlTask (void *p_arg)
{
    INT32U ctr = 0u;
    INT8U  err = OS_ERR_NONE;
                                                      /*------------------------------------------*/
    UNUSED(p_arg);
                                                      /*------------------------------------------*/
    for(;;) {
        OSSemPend(App_CtrlSem, 0u, &err);
        if (err == OS_ERR_NONE) {
            ctr += 1u;
            err = OSQPostOpt(App_CmdQ,
                             (void *)ctr,
                             OS_POST_OPT_NONE);
        }
        if (err != OS_ERR_NONE){                      /* Check for error                          */
            (void)OSMboxPostOpt(App_ErrMBox,
                                APP_ERR_PACK_CUR(APP_ERR_CTRL_TASK),
                                OS_POST_OPT_NONE);
        }
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     LED Task
* \ingroup   APP_DEMO
*
* \details   This task waits for a message in the Command Queue. The command message contains
*            a counter that is converted to a LED identifier. If the LED identifier is changed,
*            the corresponding LED is toggled.
*
* \details   This will be repeated in an endless loop.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_LedTask (void *p_arg)
{
    INT32U cmd;
    INT32U id  = HW_LED_MAX;
    INT8U  err = OS_ERR_NONE;
                                                      /*------------------------------------------*/
    UNUSED(p_arg);
                                                      /*------------------------------------------*/
    for (;;) {
        cmd = (INT32U)OSQPend(App_CmdQ, 0u, &err);
        if (err == OS_ERR_NONE) {
            cmd = (cmd / 100u) % HW_LED_MAX;
            if (id != cmd) {
                id = cmd;
                                                      /* Change temporarily to privileged mode to */
                SP_DO_PRIVIL(                         /* get write access to hardware registers   */
                    BSPPinTgl(HW_LED(id));            /* Toggle pin of current pin identifier     */
                )
            }
        }
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     UART Task
* \ingroup   APP_DEMO
*
* \details   This background task prints some diagnostic data to the serial interface:
*             - Current OS System time
*             - Data of all application tasks
*                 - Task name
*                 - Task priority
*                 - Max. task stack usage
*             - Runtime of the UART task (including interruptions from interrupt service routines
*               and other tasks with higher priority)
*
* \details   This will be repeated every 10 seconds.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_UartTask (void *p_arg)
{
    INT32U        i;
    INT8U        *taskName;
    INT8U         err;
    SP_MON_ERR_T  monErr;
    SP_MON_STK_T *mondata;
    INT32U        stkUsage;
    INT32U        osTime;
                                                      /*------------------------------------------*/
    UNUSED(p_arg);
                                                      /*------------------------------------------*/
    App_PrintString("\nApplication started.\n");
                                                      /*------------------------------------------*/
    for (;;) {
        OSTimeDly(10u * OS_TICKS_PER_SEC);            /* Delay with 10 seconds                    */

        osTime = OSTimeGet();
        App_PrintString("\n\nCurrent OS time = ");
        App_PrintUInt(osTime);
                                                      /*------------------------------------------*/
        App_PrintString("\nTask data\n  Name (Priority) / Max. Stack Usage");
        for (i = 0u; i < App_TaskPrioTblSize; i++) {
            App_PrintString("\n  ");
            OSTaskNameGet(App_TaskPrioTbl[i], &taskName, &err);
            if (err == OS_ERR_NONE) {
                App_PrintString((const char*)taskName);
            }
            App_PrintString(" (");
            App_PrintUInt(App_TaskPrioTbl[i]);
            App_PrintString(") / ");
                                                      /*------------------------------------------*/
            mondata = SPThreadStkMonGet(App_TaskPrioTbl[i]);
            monErr  = SPMonStkUsgChk(mondata, &stkUsage);
            if (monErr == SP_MON_ERR_NONE) {
                App_PrintUInt(stkUsage);
                App_PrintString("%");
            } else {
                App_PrintString("?");
            }
        }
                                                      /*------------------------------------------*/
        App_PrintString("\nUART task runtime = ");
        App_PrintUInt(OSTimeGet() - osTime);
        App_PrintString(" ticks\n");
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Print String Function
* \ingroup   APP_DEMO
*
* \details   This function handles the sending of a null terminated string using the BSP API.
*/
/*------------------------------------------------------------------------------------------------*/
void App_PrintString (const char* string)
{
    const char* pch  = string;
    INT16U      size = 0u;
    
    while (*pch != 0) {
        size++;
        pch++;
    }
   
    if (size > 0u) {
                                                      /* Change temporarily to privileged mode to */
        SP_DO_PRIVIL(                                 /* get write access to hardware registers   */
            BSPUsartSend((uint8_t *)string, size);
        )
    }
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Print Unsigned Integer Function
* \ingroup   APP_DEMO
*
* \details   This function handles the sending of unsigned numbers using the BSP API.
*/
/*------------------------------------------------------------------------------------------------*/
void App_PrintUInt (INT32U number)
{
    INT32U divisor = 1u;
    INT32U next_divisor;
    INT16U size = 0u;
    INT8U  string[12];
                                                      /*------------------------------------------*/
    while (number / divisor > 9u) {                   /* Figure out initial divisor               */
        next_divisor = divisor * 10u;
        if (next_divisor > divisor) {
            divisor = next_divisor;
        } else {
            break;
        }
    }
        do {
            string[size] = (INT8U)                    /* Now mod and print,                       */
                           ('0' + (number / divisor % 10u));
            divisor /= 10u;                           /* then divide divisor                      */
            size++;
        } while (divisor > 0u);
        
                                                      /* Change temporarily to privileged mode to */
    SP_DO_PRIVIL(                                     /* get write access to hardware registers   */
       BSPUsartSend(string, size); 
    )
}
