/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      app_hook.c
*
* \brief     Implementation of Application Hooks
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/UserCodeTemplates/app_hook.c#3 $
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  HOOKS Application Callback functions (hooks)
*
* \details   The application callback (hook) functions are called from several points within
*            the RTOS and allows to expand the RTOS functionality to suit your application.
*/
/*------------------------------------------------------------------------------------------------*/

/*
****************************************************************************************************
*                                           INCLUDES
****************************************************************************************************
*/

#include "ucos_ii.h"                                  /* use: uC/OS-II in application             */
#include "sp_core.h"                                  /* use: uC/OS-MPU extension                 */
#include "sp_osapi.h"                                 /* use: uC/OS-II wrapper functions          */
#include "RTE_Components.h"
#include CMSIS_device_header
/*
****************************************************************************************************
*                                              GLOBALS
****************************************************************************************************
*/



/*
****************************************************************************************************
*                                            LOCAL DATA
****************************************************************************************************
*/

#if (OS_CPU_HOOKS_EN > 0u) && (OS_TMR_EN > 0u) && (OS_TIME_TICK_HOOK_EN > 0u)
static INT16U OSTmrCtr;
#endif

/*
****************************************************************************************************
*                                     RTOS PORT HOOK FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  PORT_HOOKS   Application Standard Callback functions
* \ingroup   HOOKS
* @{
*/
/*------------------------------------------------------------------------------------------------*/

#if (OS_APP_HOOKS_EN > 0u)

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Task Create Hook function
*
* \details   This function is called when a task is created.
*
* \param     ptcb       pointer to the task control block of the task being created.
*
* \note      Interrupts are disabled during this call.
*/
/*------------------------------------------------------------------------------------------------*/
void App_TaskCreateHook (OS_TCB *ptcb)
{
#if OS_TMR_EN > 0u
    if (ptcb->OSTCBPrio == OS_TASK_TMR_PRIO) {
        OSTmrCtr = 0u;
    }
#else
    (void)ptcb;
#endif
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Task Delete Hook function
*
* \details   This function is called when a task is deleted.
*
* \param     ptcb       pointer to the task control block of the task being deleted.
*
* \note      Interrupts are disabled during this call.
*/
/*------------------------------------------------------------------------------------------------*/
void App_TaskDelHook (OS_TCB *ptcb)
{
    (void)ptcb;
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Idle Task Hook function
*
* \details   This function is called by the idle task.  This hook has been added to allow you to do
*            such things as STOP the CPU to conserve power.
*
* \note      Interrupts are enabled during this call.
*/
/*------------------------------------------------------------------------------------------------*/
void App_TaskIdleHook (void)
{
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Task Return Hook function
*
* \details   This function is called if a task accidentally returns. In other words, a task should
*            either be an infinite loop or delete itself when done.
*
* \param     ptcb       pointer to the task control block of the task that is returning.
*/
/*------------------------------------------------------------------------------------------------*/
void App_TaskReturnHook (OS_TCB  *ptcb)
{
    (void)ptcb;
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Statistic Task Hook function
*
* \details   This function is called every second by uC/OS-II's statistics task.  This allows your
*            application to add functionality to the statistics task.
*/
/*------------------------------------------------------------------------------------------------*/
void  App_TaskStatHook (void)
{
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application Task Switch Hook function
*
* \details   This function is called when a task switch is performed. This allows you to perform
*            other operations during a context switch.
*
* \details   It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task
*            that will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points
*            to the task being switched out (i.e. the preempted task).
*
* \note      Interrupts are disabled during this call.
*/
/*------------------------------------------------------------------------------------------------*/
#if OS_TASK_SW_HOOK_EN > 0
void  App_TaskSwHook (void)
{
}
#endif

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application TCBInit Hook function
*
* \details   This function is called by OS_TCBInit() after setting up most of the TCB.
*
* \note      Interrupts may or may not be ENABLED during this call.
*/
/*------------------------------------------------------------------------------------------------*/
void  App_TCBInitHook (OS_TCB *ptcb)
{
    (void)ptcb;
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Application TimeTick Hook function
*
* \details   This function is called every tick.
*
* \note      Interrupts may or may not be ENABLED during this call.
*/
/*------------------------------------------------------------------------------------------------*/
#if OS_TIME_TICK_HOOK_EN > 0
void  App_TimeTickHook (void)
{
#if OS_TMR_EN > 0u
    INT8U err;
                                                      /* Trigger RTOS Timer Task with configured  */
    OSTmrCtr++;                                       /* period                                   */
    if (OSTmrCtr >= (OS_TICKS_PER_SEC / OS_TMR_CFG_TICKS_PER_SEC)) {
        OSTmrCtr = 0u;
        err = OSTmrSignal();
        if (err != OS_ERR_NONE) {
            OS_SAFETY_CRITICAL_EXCEPTION();
        }
    }
#endif
}
#endif

#endif /* (OS_APP_HOOKS_EN > 0u) */

/*! @}*/

/*
****************************************************************************************************
*                                   EXCEPTION/FAULT HOOK FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \defgroup  FAULT_HOOKS   Application Fault Callback functions
* \ingroup   HOOKS
* @{
*/
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Safety Critical Exception Callback
*
* \details   This function is called from the RTOS to enter a safe-state.
*
* \details   Here you can implement your reaction on internal panic errors.
*            During development there may be a chance to reach this function so some kind
*            of diagnostic information is helpful.
*            In production environment, this function is most likely reached after
*            a SEU error somewhere.
*
* \note      This task function is executed in privileged or non-privileged mode.
*/
/*------------------------------------------------------------------------------------------------*/
void OS_SAFETY_CRITICAL_EXCEPTION (void)
{
    volatile INT8U cont = 0u;

    SP_DO_PRIVIL(                                     /* Change to privileged thread mode         */
        __disable_irq();                              /*   to disable all interrupts              */
    )
    while (!cont);                                    /* stop for debugging; set cont to 1 to go  */
                                                      /* back to the errornous location           */
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Memory Access Fault Event Callback
*
* \details   This function is called if a write-access violation occurred.
*
* \details   Here you can implement your reaction on memory access errors.
*            During development there may be a chance to reach this function so some kind of
*            diagnostic information is helpful.
*            In production environment, this function is most likely reached after
*            a SEU error somewhere.
*
* \note      This task function is executed in privileged mode.
*/
/*------------------------------------------------------------------------------------------------*/
void SPAccessFaultEvent (SP_ACCESS_FAULT_T *err)
{
    volatile INT8U cont = 0u;

    __disable_irq();                                  /* Disable all interrupts                   */

    while (!cont);                                    /* Stop for debugging; set cont to 1 to     */
                                                      /* suspend bad task, other value to the     */
                                                      /* instruction, which does the bad access   */
    if (cont == 1u) {
        OSTaskSuspend(err->CurPrio);
    }
    err->CurPrio  = 0uL;
    err->IAddress = 0uL;
    err->DAddress = 0uL;
}

/*! @}*/
