/*
****************************************************************************************************
* (c) copyright by
*     Embedded Office GmbH & Co. KG       Tel : (07522) 97 00 08-0
*     Friedrich-Ebert-Str. 20/1           Fax : (07522) 97 00 08-99
*     D-88239 Wangen                      Mail: info@embedded-office.de
*                                         Web : http://www.embedded-office.de
*
* All rights reserved. Confidential and Proprietary. Protected by international copyright laws.
* Knowledge of the source code may not be used to write a similar product.
* This file may only be used in accordance with a license and should not be
* redistributed in any way.
****************************************************************************************************
*/
/*!
****************************************************************************************************
* \file      main.c
*
* \brief     Flexible Safety RTOS Main function template
*
* $Id: //stream_demos/demo_cmsis_pack/demo_cmsis/Files/UserCodeTemplates/main.c#2 $
*
****************************************************************************************************
*/
/*----------------------------------------END OF HEADER-------------------------------------------*/

/*
****************************************************************************************************
*                                             INCLUDES
****************************************************************************************************
*/

#include "main.h"
//TODO To install and start the OS System Time Tick using the "rtos_excep.c" template file include header file #include "rtos_excep.h" */ 

/*
****************************************************************************************************
*                                              GLOBALS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Stack memory for Start Task
* \details   The Start Task stack is located to a named memory area using the macro SP_MEM(10).
*/
/*------------------------------------------------------------------------------------------------*/
SP_MEM(10) OS_STK    App_StartStk[APP_START_STK_SZ];

SP_MEM_DECLARE(10);                                   /* Get section memory labels from linker    */

/*
****************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
****************************************************************************************************
*/

static void App_StartTask   (void *p_arg);
static void App_TaskCreate  (void (*task)(void *p_arg),
                             OS_STK stk[], INT8U prio, INT32U stkSize, void *phdl,
                             INT8U *pname);


/*
****************************************************************************************************
*                                        STARTUP FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Main Entry Function
* \ingroup   APP_STARTUP
*
* \details   The entry function is responsible for the uC/OS-MPU system startup:
*             1. Perform low level and CPU initialization.
*             2. Initialize uC/OS-MPU system by calling OSInit().
*             3. Create process for application "Start Task".
*             4. Create the application "Start Task" with highest priority.
*             5. Start uC/OS-MPU system by calling OSStart().
*/
/*------------------------------------------------------------------------------------------------*/
int main (void)
{
    SP_PROCESS_T *phdl;                               /* Local: process handle                    */
                                                      /*------------------------------------------*/
    // TODO Initialize Hardware Essential Parts

    OSInit();                                         /* initialize uC/OS-MPU realtime kernel     */
    
                                                      /* Enable MemManage fault exceptions        */
    SCB->SHCSR |= (SCB_SHCSR_MEMFAULTENA_Msk |
                   SCB_SHCSR_BUSFAULTENA_Msk |
                   SCB_SHCSR_USGFAULTENA_Msk);
                                                      /*------------------------------------------*/
    phdl = SPProcessCreate(SP_MEM_START(10),          /* Create process space for stack memory    */
                           SP_MEM_END(10));

    App_TaskCreate(App_StartTask,                     /* Create Application Start Task            */
                   App_StartStk,
                   APP_START_PRIO,
                   APP_START_STK_SZ,
                   (void *)phdl,
                   (INT8U *)"Start Task");

    OSStart();                                        /* Start uC/OS-MPU realtime kernel          */
}

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     High Priority Application Start Task
* \ingroup   APP_STARTUP
*
* \details   This task is the first task (with the highest task priority) running after OSStart().
*            This task
*             1. Do the high level BSP initialization.
*             2. Change to non-privileged (user) mode.
*             3. Create RTOS resources used in this application.
*             4. Call OSSafetyCriticalStart() to indicate that all initialization has been
*                completed and that kernel objects are no longer allowed to be created.
*             5. Install and start a hardware timer for the OS System Time Tick.
*                Note, to prevent the usage of a RTOS event handle in App_TimeTickHook() before
*                it is created, the OS System Time Tick timer is started only after completed
*                initialization.
*
* \note      This task function is executed in privileged mode until SP_SET_USR_MODE() is called.
*/
/*------------------------------------------------------------------------------------------------*/
static void App_StartTask (void *p_arg)
{
                                                      /* TASK IS RUNNING IN PRIVILEGED MODE       */
    // TODO Initialize BSP functions
                                                      /*------------------------------------------*/
    SP_SET_USR_MODE();                                /* Switch to non-privileged (user) mode     */
                                                      /* TASK IS RUNNING IN NON-PRIVILEGED MODE   */
                                                      /*------------------------------------------*/
    // TODO Create application tasks and events

                                                      /*------------------------------------------*/
    OSSafetyCriticalStart();                          /* Indicate completed initialization        */

    SP_DO_PRIVIL(
        // TODO Install and Start OS System Time Tick
        /* User can use "rtos_excep.c" template file to install and start the OS System Time Tick 
           and call here function RTOS_Tick_Init() */
    )
                                                      /*------------------------------------------*/
    for (;;) {                                        /* Endless Task Loop                        */
        OSTimeDly(OS_TICKS_PER_SEC);
    }
}

/*
****************************************************************************************************
*                                      APPLICATION FUNCTIONS
****************************************************************************************************
*/

/*------------------------------------------------------------------------------------------------*/
/*!
* \brief     Helper function to create a RTOS task
* \ingroup   APP_DEMO
*
* \details   This function is used to simplify the creation of an application task. For this
*            demo application a few task parameter are not used or can be set to fixed values:
*             - p_arg : not used, set to 0
*             - ptos, pbos : calculated depending on the stack growth, the given stack array and
*               the given stack size
*             - id : not used, set to the priority value
*             - opt : set to OS_TASK_OPT_NONE
*
* \param     task      Pointer to the task's code
* \param     stk       Task's stack memory array
* \param     prio      Task's priority.
* \param     stkSize   Size of the stack in number of OS_STK elements
* \param     phdl      Process handle the task will be assigned to.
* \param     pname     ASCII string that contains the name of the task
*/
/*------------------------------------------------------------------------------------------------*/
static void App_TaskCreate (void (*task)(void *p_arg),
                            OS_STK stk[], INT8U prio, INT32U stkSize, void *phdl,
                            INT8U *pname)
{
    INT8U   err;
    OS_STK *ptos;
    OS_STK *pbos;
                                                      /*------------------------------------------*/
#if OS_STK_GROWTH == 1                                /* Calculate the stack boundaries depending */
    ptos = &stk[stkSize - 1];                         /* on the stack growth and the stack size   */
    pbos = &stk[0];
#else
    ptos = &stk[0];
    pbos = &stk[stkSize - 1];
#endif
                                                      /*------------------------------------------*/
    err = OSTaskCreateExt(task,
                          (void *)0,
                          ptos,
                          prio,
                          prio,
                          pbos,
                          stkSize,
                          phdl,
                          OS_TASK_OPT_NONE );

    if (err != OS_ERR_NONE) {                         /* see, if task can't be created            */
        OS_SAFETY_CRITICAL_EXCEPTION();               /* Fatal Error: stop CPU in safety state    */
    }

#if (OS_TASK_NAME_EN > 0u)
    OSTaskNameSet(prio, pname, &err);                 /* Set the task's name                      */
#endif
}
